class Information(object):
    """ Class definition of the device information. """

    def __init__(self, id, manufacturer, model, firmware, hardware, batch):
        """ Set the private variable values on instantiation. """
        self.__id = id
        self.__manufacturer = manufacturer
        self.__model = model
        self.__firmware = firmware
        self.__hardware = hardware
        self.__batch = batch

    def __str__(self):
        """ Define how the print() method should print the object. """
        object_type = str(type(self))
        return object_type + ": " + str(self.as_dict())

    def __repr__(self):
        """ Define how the object is represented on output to console. """
        class_name   = type(self).__name__
        id           = f"id = '{self.id}'"
        manufacturer = f"manufacturer = '{self.manufacturer}'"
        model        = f"model = '{self.model}'"
        firmware     = f"firmware = '{self.firmware}'"
        hardware     = f"hardware = {self.hardware}"
        batch        = f"batch = '{self.batch}'"

        return f"{class_name}({id}, {manufacturer}, {model}, {firmware}, {hardware}, {batch})"

    def as_dict(self):
        """ Return the object properties in a dictionary. """
        return {
            'id': self.id,
            'manufacturer': self.manufacturer,
            'model': self.model,
            'firmware': self.firmware,
            'hardware': self.hardware,
            'batch': self.batch,
        }

    # Information properties
    @property
    def id(self):
        return self.__id

    @property
    def manufacturer(self):
        return self.__manufacturer

    @property
    def model(self):
        return self.__model

    @property
    def firmware(self):
        return self.__firmware

    @property
    def hardware(self):
        return self.__hardware

    @property
    def batch(self):
        return self.__batch


class Electricity(object):
    """ Class definition of a set of power measurements. """

    def __init__(self, now, minimum, maximum, imported, exported):
        """ Set the private variable values on instantiation. """
        self.__now = now
        self.__minimum = minimum
        self.__maximum = maximum
        self.__imported = imported
        self.__exported = exported

    def __str__(self):
        """ Define how the print() method should print the object. """
        object_type = str(type(self))
        return object_type + ": " + str(self.as_dict())

    def __repr__(self):
        """ Define how the object is represented on output to console. """
        class_name = type(self).__name__
        now      = f"now = '{self.now}'"
        minimum       = f"minimum = '{self.minimum}'"
        maximum  = f"maximum = '{self.maximum}'"
        imported  = f"imported = '{self.imported}'"
        exported  = f"exported = '{self.exported}'"

        return f"{class_name}({now}, {minimum}, {maximum}, {imported}, {exported})"

    def as_dict(self):
        """ Return the object properties in a dictionary. """
        return {
            'now': self.now,
            'minimum': self.minimum,
            'maximum': self.maximum,
            'imported': self.imported,
            'exported': self.exported,
        }

    # Electricity properties
    @property
    def now(self):
        return self.__now

    @property
    def minimum(self):
        return self.__minimum

    @property
    def maximum(self):
        return self.__maximum

    @property
    def imported(self):
        return self.__imported

    @property
    def exported(self):
        return self.__exported



class Power(object):
    """ Class definition of a power measurement (value, unit and timestamp). """

    def __init__(self, key, value, unit, timestamp):
        """ Set the private variable values on instantiation. """
        self.__key = key
        self.__value = value
        self.__unit = unit
        self.__timestamp = timestamp

    def __str__(self):
        """ Define how the print() method should print the object. """
        object_type = str(type(self))
        return object_type + ": " + str(self.as_dict())

    def __repr__(self):
        """ Define how the object is represented on output to console. """
        class_name = type(self).__name__
        key        = f"key = '{self.key}'"
        value      = f"value = {self.value}"
        unit       = f"unit = '{self.unit}'"
        timestamp  = f"timestamp = '{self.timestamp}'"

        return f"{class_name}({key}, {value}, {unit}, {timestamp})"

    def as_dict(self):
        """ Return the object properties in a dictionary. """
        return {
            'key': self.key,
            'value': self.value,
            'unit': self.unit,
            'timestamp': self.timestamp,
        }

    # Power properties
    @property
    def key(self):
        return self.__key

    @property
    def value(self):
        return self.__value

    @property
    def unit(self):
        return self.__unit

    @property
    def timestamp(self):
        return self.__timestamp


class WLANInformation(object):
    """ Class definition of the device WLAN information. """

    def __init__(self, ap_key, ap_mac, ap_ssid, client_ssid, dns, dnsalt,
                 eth_mac, gateway, ip, join_status, mac, mode, n2g_id, sta_mac, subnet):
        """ Set the private variable values on instantiation. """
        self.__ap_key = ap_key
        self.__ap_mac = ap_mac
        self.__ap_ssid = ap_ssid
        self.__client_ssid = client_ssid
        self.__dns = dns
        self.__dnsalt = dnsalt
        self.__eth_mac = eth_mac
        self.__gateway = gateway
        self.__ip = ip
        self.__join_status = join_status
        self.__mac = mac
        self.__mode = mode
        self.__n2g_id = n2g_id
        self.__sta_mac = sta_mac
        self.__subnet = subnet

    def __str__(self):
        """ Define how the print() method should print the object. """
        object_type = str(type(self))
        return object_type + ": " + str(self.as_dict())

    def __repr__(self):
        """ Define how the object is represented on output to console. """
        class_name  = type(self).__name__
        ap_key      = f"ap_key = '{self.ap_key}'"
        ap_mac      = f"ap_mac = '{self.ap_mac}'"
        ap_ssid     = f"ap_ssid = '{self.ap_ssid}'"
        client_ssid = f"client_ssid = '{self.client_ssid}'"
        dns         = f"dns = '{self.dns}'"
        dnsalt      = f"dnsalt = '{self.dnsalt}'"
        eth_mac     = f"eth_mac = '{self.eth_mac}'"
        gateway     = f"gateway = '{self.gateway}'"
        ip          = f"ip = '{self.ip}'"
        join_status = f"join_status = {self.join_status}"
        mac         = f"mac = '{self.mac}'"
        mode        = f"mode = '{self.mode}'"
        n2g_id      = f"n2g_id = '{self.n2g_id}'"
        sta_mac     = f"sta_mac = '{self.sta_mac}'"
        subnet      = f"subnet = '{self.subnet}'"

        return f"{class_name}({ap_key}, {ap_mac}, {ap_ssid}, {client_ssid}, {dns}, {dnsalt}, {eth_mac}, {gateway}, {ip}, {join_status}, {mac}, {mode}, {n2g_id}, {sta_mac}, {subnet})"

    def as_dict(self):
        """ Return the object properties in a dictionary. """
        return {
            'ap_key': self.ap_key,
            'ap_mac': self.ap_mac,
            'ap_ssid': self.ap_ssid,
            'client_ssid': self.client_ssid,
            'dns': self.dns,
            'dnsalt': self.dnsalt,
            'eth_mac': self.eth_mac,
            'gateway': self.gateway,
            'ip': self.ip,
            'join_status': self.join_status,
            'mac': self.mac,
            'mode': self.mode,
            'n2g_id': self.n2g_id,
            'sta_mac': self.sta_mac,
            'subnet': self.subnet,
        }

    # Power properties
    @property
    def ap_key(self):
        return self.__ap_key

    @property
    def ap_mac(self):
        return self.__ap_mac

    @property
    def ap_ssid(self):
        return self.__ap_ssid

    @property
    def client_ssid(self):
        return self.__client_ssid

    @property
    def dns(self):
        return self.__dns

    @property
    def dnsalt(self):
        return self.__dnsalt

    @property
    def eth_mac(self):
        return self.__eth_mac

    @property
    def gateway(self):
        return self.__gateway

    @property
    def ip(self):
        return self.__ip

    @property
    def join_status(self):
        return self.__join_status

    @property
    def mac(self):
        return self.__mac

    @property
    def mode(self):
        return self.__mode

    @property
    def n2g_id(self):
        return self.__n2g_id

    @property
    def sta_mac(self):
        return self.__sta_mac

    @property
    def key(self):
        return self.__key

    @property
    def subnet(self):
        return self.__subnet





import json
import requests

from datetime import datetime

from elna.exceptions import *

class API(object):
    """ Class used for communication with the SmartMeter API """

    # Client configuration
    __app_type = 'com.elna.smartmeter.client'
    __user_agent = 'Dart/2.10 (dart:io)'

    # Use a session to reuse one TCP connection instead of creating a new
    # connection for every call to the API
    __session = None
    __timeout = 4

    def __init__(self, hostname):
        """ Class constructor initializes all URL variables. """

        # Set connection specific details
        self.__hostname = hostname

        self.__url_base = 'http://' + self.__hostname

        # API endpoints
        self.__url_info         = self.__url_base + '/info'
        self.__url_meter_now    = self.__url_base + '/meter/now'
        self.__url_wlan_info    = self.__url_base + '/wlan/info'

        # Create a new session
        self.__session = requests.session()

    def __send_request(self, url, with_session_token=True, with_user_token=True, data_json=None, request_type='GET'):
        """ Send a GET or POST request to the server. """

        # Prepare the headers to be sent
        headers = {
            'Host': self.__hostname,
            'Connection': 'keep-alive',
            'Accept': '*/*',
            'User-Agent': self.__user_agent,
        }

        # Only needed for POST requests
        if request_type == 'POST':
            headers['Content-Type'] = 'application/json'
            headers['Content-Length'] = str(len(data_json))

        # Perform the request and raise an exception
        # if the response is not OK (HTML 200)
        try:
            if request_type == 'GET':
                response = self.__session.get(url, headers=headers, timeout=self.__timeout)
            elif request_type == 'POST':
                response = self.__session.post(url, headers=headers, data=data_json, timeout=self.__timeout)
            response.raise_for_status()
        except requests.exceptions.ConnectionError:
            raise NewConnectionError(f"Failed to establish a new connection to '{url}'.")
            return None
        except requests.exceptions.ConnectTimeout:
            raise ConnectionTimeoutError(f"Connection to '{self.__hostname}' timed out after {str(self.__timeout)} seconds.")
            return None
        except:
            raise

        # Check HTTP response code
        if response.status_code == requests.codes.ok:
            return json.loads(response.content.decode('utf-8'))
        else:
            return None

    ######################
    # Public API methods #
    ######################

    @property
    def hostname(self):
        """ Property to keep track of the API servers hostname. """
        return self.__hostname

    def get_info(self):
        """ Get the general information about the device. """
        return self.__send_request(self.__url_info, request_type='GET')

    def get_meter_now(self):
        """ Get the power statistics of the device. """
        return self.__send_request(self.__url_meter_now, request_type='GET')

    def get_wlan_info(self):
        """ Get the WLAN information of the device. """
        return self.__send_request(self.__url_wlan_info, request_type='GET')

    def send_get(self, url):
        """ Send a custom POST request. """
        return self.__send_request(url, request_type='GET')

    def send_post(self, url, data):
        """ Send a custom POST request. """
        data_json = json.dumps(data, separators=(',', ':'))
        return self.__send_request(url, data_json=data_json, request_type='POST')



# define Python user-defined exceptions
class Error(Exception):
    """ Base class for other exceptions """
    pass


class NewConnectionError(Error):
    """ Raised when failing to establish a new connection to the device. """
    def __init__(self, message="Failed to establish a new connection."):
        self.message = message
        super().__init__(self.message)




from datetime import datetime

from elna.core import API
from elna.exceptions import *
from elna.classes import *


class Connect(object):
    """ Class definition of the Elna SmartMeter. """

    # API Connection
    __api = None

    def __init__(self, hostname):
        """ Initiate the connection to the API. """
        self.__api = API(hostname)

    # System properties
    @property
    def api(self):
        """ Return the API for direct access. """
        return self.__api

    def get_info(self):
        """ Fetch general information from the device. """
        info = self.__api.get_info()

        new_info = Information(
            id=info['id'],
            manufacturer=info['mf'],
            model=info['model'],
            firmware=info['fw'],
            hardware=info['hw'],
            batch=info['batch']
        )

        return new_info

    def get_electricity(self):
        """ Fetch power readings from SmartMeter and populate class objects. """
        power = self.__api.get_meter_now()

        # Power Readings
        power_now = Power(
            key='now',
            value=power['elec']['power']['now']['value'],
            unit=power['elec']['power']['now']['unit'],
            timestamp=datetime.fromtimestamp(power['elec']['power']['now']['time'])
        )

        power_min = Power(
            key='minimum',
            value=power['elec']['power']['min']['value'],
            unit=power['elec']['power']['min']['unit'],
            timestamp=datetime.fromtimestamp(power['elec']['power']['min']['time'])
        )

        power_max = Power(
            key='maximum',
            value=power['elec']['power']['max']['value'],
            unit=power['elec']['power']['max']['unit'],
            timestamp=datetime.fromtimestamp(power['elec']['power']['max']['time'])
        )

        power_import = Power(
            key='imported',
            value=power['elec']['import']['now']['value'],
            unit=power['elec']['import']['now']['unit'],
            timestamp=datetime.fromtimestamp(power['elec']['import']['now']['time'])
        )

        power_export = Power(
            key='exported',
            value=power['elec']['export']['now']['value'],
            unit=power['elec']['export']['now']['unit'],
            timestamp=datetime.fromtimestamp(power['elec']['export']['now']['time'])
        )

        electricity = Electricity(power_now, power_min, power_max, power_import, power_export)

        return electricity

    def get_wlan_info(self):
        """ Fetch WLAN information from the device. """
        info = self.__api.get_wlan_info()

        new_info = WLANInformation(
            ap_key=info['ap_key'],
            ap_mac=info['ap_mac'],
            ap_ssid=info['ap_ssid'],
            client_ssid=info['client_ssid'],
            dns=info['dns'],
            dnsalt=info['dnsalt'],
            eth_mac=info['eth_mac'],
            gateway=info['gateway'],
            ip=info['ip'],
            join_status=info['join_status'],
            mac=info['mac'],
            mode=info['mode'],
            n2g_id=info['n2g_id'],
            sta_mac=info['sta_mac'],
            subnet=info['subnet']
        )

        return new_info


